<?php

namespace Drupal\cmlapi\Service;

/**
 * Scheme from 1c.
 */
class Scheme {

  /**
   * Svoistvos.
   */
  protected array $svoistvo;

  /**
   * Taxonomy.
   */
  protected array $taxonomy;

  /**
   * Catalog Parser.
   *
   * @var \Drupal\cmlapi\Service\ParserCatalog
   */
  protected $catalogParcer;

  /**
   * Xml Parser.
   *
   * @var \Drupal\cmlapi\Service\XmlParser
   */
  protected $xmlParser;

  /**
   * Constructs a new Scheme object.
   *
   * @param \Drupal\cmlapi\Service\ParserCatalog $catalog
   *   CAtalog Parser.
   * @param \Drupal\cmlapi\Service\XmlParser $xml_parser
   *   XML Parcer.
   */
  public function __construct(
    ParserCatalog $catalog,
    XmlParser $xml_parser
  ) {
    $this->svoistvo = [];
    $this->taxonomy = [];
    $this->catalogParcer = $catalog;
    $this->xmlParser = $xml_parser;
  }

  /**
   * Page tree.
   */
  public function init(int $cml) {
    $data = $this->catalogParcer->parse($cml);
    if (\Drupal::moduleHandler()->moduleExists('devel')) {
      // dsm($data);
    }
    $this->svoistvoList($data);
    $category = $this->categoryList($data);
    return [
      'category' => $category,
      'svoistvo' => $this->svoistvo,
      'taxonomy' => $this->taxonomy,
    ];
  }

  /**
   * Category.
   */
  private function categoryList($data) {
    $category = [];
    if ($data && isset($data['category']) && $data['category']) {
      foreach ($data['category'] as $id => $cat) {
        $category[$id] = [
          'id' => $id,
          'name' => $cat['Наименование'],
          'svoistva' => $this->categoryProps($cat),
        ];
      }
    }
    return $category;
  }

  /**
   * Props.
   */
  private function categoryProps($cat) {
    $props = [];
    if (!empty($cat['Свойства']['Ид'])) {
      $pids = $this->xmlParser->arrayNormalize($cat['Свойства']['Ид']);
      foreach ($pids as $pid) {
        if (isset($this->svoistvo[$pid])) {
          $props[$pid] = $this->svoistvo[$pid];
        }
        else {
          $props[$pid] = $pid;
        }
      }
    }
    return $props;
  }

  /**
   * Svoistvos.
   */
  private function svoistvoList($data) {
    if ($data && !empty($data['svoistvo'])) {
      $svoistvos = $this->xmlParser->arrayNormalize($data['svoistvo']);
      foreach ($svoistvos as $value) {
        $id = $value['Ид'];
        $type = $value['ТипЗначений'];
        $product = [];
        if (isset($value['ДляТоваров'])) {
          $product = $value['ДляТоваров'];
        }
        $svoistvo = [
          'id' => $id,
          'name' => $value['Наименование'],
          'type' => $value['ТипЗначений'],
          'product' => $product,
          'props' => $this->svoistvoProps($value),
        ];
        if ($type == 'Справочник') {
          $this->taxonomy[$id] = $svoistvo;
        }
        else {
          $this->svoistvo[$id] = $svoistvo;
        }
      }
    }
  }

  /**
   * Svoistvo Props.
   */
  private function svoistvoProps($svoistvo) {
    $props = [];
    if (!empty($svoistvo['ВариантыЗначений']['Справочник'])) {
      $props = $this->xmlParser->xml2KeyVal($svoistvo['ВариантыЗначений']['Справочник']);
    }
    return $props;
  }

}
